package ca.canadapost.cpcdp.pickup;

import ca.canadapost.cpcdp.pickup.generated.messages.Messages;
import ca.canadapost.cpcdp.pickup.generated.pickup.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the GetPickupAvailability Canada Post service.
 * <p>
 * The GetPickupAvailability service identifies if pickup services 
 * are available for a given Postal Code. This call also provides information 
 * related to the type of pickup available and the cut-off times for pickup service 
 * requests. 
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 * Jersey REST client using JAXB for GetPickupAvailability Request.  <br>
 *  USAGE:<pre>
 *        GetPickupAvailabilityClient client = new GetPickupAvailabilityClient(username, password);
 *        ClientResponse response = client.getPickupAvailability();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 */
public class GetPickupAvailabilityClient {

    private Client aClient;
    private static final String LINK = "https://ct.soa-gw.canadapost.ca/ad/pickup/pickupavailability/K2B8J6";
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username and password are imported from the following file
    	// CPCWS_Pickup_Java_Samples/user.properties
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
						
		// Execute GetPickupAvailability Request
        GetPickupAvailabilityClient myClient = new GetPickupAvailabilityClient(username, password);
        ClientResponse resp = myClient.getPickupAvailability();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(PickupAvailability.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches PickupAvailability schema.
            if (entity instanceof PickupAvailability) {
            	PickupAvailability pickup = (PickupAvailability) entity;           	
                System.out.println("Postal Code: " + pickup.getPostalCode());
                System.out.println("On Demand Tour: " + pickup.getOnDemandTour());
                System.out.println("On Demand Cut Off: " + pickup.getOnDemandCutoff());
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public GetPickupAvailabilityClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getPickupAvailability() throws UniformInterfaceException {    	
    	WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.pickup+xml").acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
